package com.example.testpreview;

import java.io.IOException;

import android.content.Context;
import android.graphics.Bitmap;
import android.hardware.Camera;
import android.hardware.Camera.PreviewCallback;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;

public class CameraPreview extends SurfaceView implements SurfaceHolder.Callback 
{
    SurfaceHolder mHolder;
    Camera mCamera;
    DrawPreview mDrawPreview;
    boolean mFinished;

    CameraPreview(Context context, DrawPreview drawOnTop) 
    {
        super(context);
        
        mDrawPreview = drawOnTop;
        mFinished = false;

        // Install a SurfaceHolder.Callback so we get notified when the
        // underlying surface is created and destroyed.
        mHolder = getHolder();
        mHolder.addCallback(this);
        //mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
    }

    public void surfaceCreated(SurfaceHolder holder) {
        mCamera = Camera.open();
        try {
           mCamera.setPreviewDisplay(holder);
           
           // Preview callback used whenever new frame is available
           mCamera.setPreviewCallback(new PreviewCallback() {
        	  public void onPreviewFrame(byte[] data, Camera camera)
        	  {
        		  if ( (mDrawPreview == null) || mFinished )
        			  return;
        		  
        		  if (mDrawPreview.mBitmap == null)
        		  {
        			  // Initialize the draw-on-top companion
        			  Camera.Parameters params = camera.getParameters();
        			  mDrawPreview.mImageWidth = params.getPreviewSize().width;
        			  mDrawPreview.mImageHeight = params.getPreviewSize().height;
        			  mDrawPreview.mBitmap = Bitmap.createBitmap(mDrawPreview.mImageWidth, mDrawPreview.mImageHeight, Bitmap.Config.ARGB_8888);
        			  mDrawPreview.mRGBData = new int[mDrawPreview.mImageWidth * mDrawPreview.mImageHeight]; 
        			  mDrawPreview.mYUVData = new byte[data.length];        			  
        		  }
        		  
        		  // Pass YUV data to draw-on-top companion
        		  System.arraycopy(data, 0, mDrawPreview.mYUVData, 0, data.length);
        		  mDrawPreview.invalidate();
        	  }
           });
           
	        // Define on touch listener
           this.setOnTouchListener(new OnTouchListener() 
           {
	   			public boolean onTouch(View v, MotionEvent event)
	   			{
	   				if (mDrawPreview.mState == DrawPreview.STATE_ORIGINAL)
	   				{
	   					mDrawPreview.mState = DrawPreview.STATE_PROCESSED;
	   				}
	   				else if (mDrawPreview.mState == DrawPreview.STATE_PROCESSED)
	   				{
	   					mDrawPreview.mState = DrawPreview.STATE_ORIGINAL;
	   				}
	   				return false;
	   			}
	   		});
        } 
        catch (IOException exception) {
            mCamera.release();
            mCamera = null;
        }
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        // Surface will be destroyed when we return, so stop the preview.
        // Because the CameraDevice object is not a shared resource, it's very
        // important to release it when the activity is paused.
    	mFinished = true;
    	mCamera.setPreviewCallback(null);
        mCamera.stopPreview();
        mCamera.release();
        mCamera = null;
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
        // Now that the size is known, set up the camera parameters and begin
        // the preview.
        Camera.Parameters parameters = mCamera.getParameters();
        parameters.setSceneMode(Camera.Parameters.SCENE_MODE_NIGHT);
        parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
        mCamera.setParameters(parameters);
        mCamera.startPreview();
    }
}